"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = loader;

var _loaderUtils = require("loader-utils");

var _schemaUtils = _interopRequireDefault(require("schema-utils"));

var _compress = _interopRequireDefault(require("./lib/compress"));

var _inline = _interopRequireDefault(require("./lib/inline"));

var _emit = _interopRequireDefault(require("./lib/emit"));

var _constants = require("./constants");

var _options = _interopRequireDefault(require("./options.json"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * @file: main entry
 * @author: GaoYYYang
 * @date: 06 30 2020 4:51:22
 */
function checkNeedInline(option, size) {
  if (option.disable) {
    return false;
  }

  const {
    resourceQuery
  } = this;
  const {
    limit,
    symbol,
    antiSymbol
  } = option;
  const query = resourceQuery && (0, _loaderUtils.parseQuery)(resourceQuery) || {};

  if (query[symbol]) {
    return true;
  }

  if (query[antiSymbol]) {
    return false;
  }

  if (typeof limit === 'boolean') {
    return limit;
  }

  if (typeof limit === 'string') {
    return size <= parseInt(limit, 10);
  }

  if (typeof limit === 'number') {
    return size <= limit;
  }

  return true;
}

function inlineOrEmit(inlineOption, data) {
  if (checkNeedInline.call(this, inlineOption, data.length)) {
    _inline.default.call(this, data, inlineOption);
  } else {
    _emit.default.call(this, data, inlineOption);
  }
}

function loader(source) {
  const options = (0, _loaderUtils.getOptions)(this) || {};
  const {
    mode,
    resourcePath,
    resourceQuery
  } = this;
  (0, _schemaUtils.default)(_options.default, options, 'image-optimize-loader');
  let compressOption = _constants.LOSELESS_COMPRESS_OPTION;

  if (options.compress) {
    if (options.compress.mode === 'lossy') {
      compressOption = Object.assign(_constants.LOSSY_COMPRESS_OPTION, options.compress);
    } else {
      compressOption = Object.assign(compressOption, options.compress);
    }
  }

  let inlineOption = _constants.INLINE_OPTION;

  if (options.inline) {
    inlineOption = Object.assign(inlineOption, options.inline);
  }

  if (mode === 'production' || compressOption.bypassOnDebug === false || !compressOption.disable) {
    const callback = this.async();
    (0, _compress.default)(source, compressOption).then(data => {
      inlineOrEmit.call(this, inlineOption, data);
    }).catch(err => {
      callback(err);
    });
  } else {
    inlineOrEmit.call(this, inlineOption, source);
  }
}

module.exports.raw = true;